

#include "environ.h"
#include "ray.h"
#include "types.h"
#include <GL/glut.h>
// for isosurface triangulation:
#include "gts.h"

extern int environ_type;

#define NORMAL_E 1e-4

const FLT Environ::maxgrad = 2;

FLT Environ::inside(const Vector3& l) const {

	// the isosurface function goes here:
	FLT x = l[0];
	FLT y = l[1];
	FLT z = l[2];
	
	switch(environ_type) {
	
		default:
		case ENVIRON_XYZ:
			return std::min( y+5 , std::min( -z+30 , -x+12) );
			
		case ENVIRON_SINX:
			return std::min( -x+14.f, (float)sin(x+2.f)+y+5.f );
	
		case ENVIRON_SINXSINZ:
			return std::min( -x+14.f, (float)(sin(x)*sin(z)+y+5) );
			
		case ENVIRON_SINXXZZ:
			return std::min( (float)(0.8f*sin(sqrt(2.f*x*x+2.*(z-23.f)*(z-23.f)))+y+5.f), -x+12.f );
	
	}
	
//	FLT fsin = sin(x)*sin(z)+y+5;
//	return std::min( std::min( y+5,-(x*0.8f+z*0.2f)+12.0f ), x+1);
//	return std::min( -x+12, fsin );
//	return y+4;
//	return std::min( -x+12, (float)(-0.001*x*x-0.03*(z-21)*(z-23)+y+5) );
//	return std::min( (float)(0.8f*sin(sqrt(2.f*x*x+2.*(z-23.f)*(z-23.f)))+y+5.f), -x+12.f );


/*	FLT x = l[0]-5;
	FLT y = l[1];
	FLT z = l[2]-20;
	
	return -(sqrt(x*x+y*y+z*z)-9);*/

}

Vector3 Environ::gradient(const Vector3& location) const {

        DBL f0 = inside(location);
        DBL dx = inside(Vector3::add(location,Vector3(NORMAL_E,0,0))) - f0;
        DBL dy = inside(Vector3::add(location,Vector3(0,NORMAL_E,0))) - f0;
        DBL dz = inside(Vector3::add(location,Vector3(0,0,NORMAL_E))) - f0;
	
	Vector3 ret(dx,dy,dz);
	ret.normalize();
	return ret;

}

void Environ::project(Vector3& p) const {

	// set up a ray
	Ray projector(p,gradient(p));

	int oopses = 0;
	// find intersection of p with the surface
	FLT t = 0;
	FLT penlast = -100;
	do {
	
		FLT pen = inside(projector.positionAtTime(t));
		if (pen < penlast) {
			Vector3 pnew = projector.positionAtTime(t);			
			projector = Ray(pnew,gradient(pnew));			
//			std::cout << "oops";
			if (++oopses > 10) return;
		}
		penlast = pen;
//		std::cout << p << " t = " << t << ", " << pen<<std::endl;
		if (pen>-INTERSECTION_TIME_EPSILON) break;
		t = t - pen/maxgrad;		
			
	} while(1);
	
	p = projector.positionAtTime(t+2*INTERSECTION_TIME_EPSILON);	

}

void Environ::gtsFunction(gdouble ** f, GtsCartesianGrid g, guint k, gpointer env) {

  gdouble x, y, z = g.z;
  guint i, j;

  for (i = 0, x = g.x; i < g.nx; i++, x += g.dx)
    for (j = 0, y = g.y; j < g.ny; j++, y += g.dy)
      f[i][j] = ((Environ*)env)->inside(Vector3(x,y,z));

}

static void print_vertex (GtsFace * face, void ** null)
{

  Environ* env = (Environ*)null;

//  GLdouble x,y,z;
  GtsVertex *A,*B,*C;  
//  gts_triangle_normal(&(face->triangle),&x,&y,&z);
  gts_triangle_vertices(&(face->triangle),&A,&B,&C);

//  Vector3 n(x,y,z);
//  if (n.dotProd(gradient(Vector3(A

//  std::cout << Vector3(  A->p.x,A->p.y,A->p.z);
  glNormal3fv(env->gradient(Vector3(A->p.x,A->p.y,A->p.z)).glFloat());    
  glVertex3f(A->p.x,A->p.y,A->p.z);
  glNormal3fv(env->gradient(Vector3(B->p.x,B->p.y,B->p.z)).glFloat());      
  glVertex3f(B->p.x,B->p.y,B->p.z);
  glNormal3fv(env->gradient(Vector3(C->p.x,C->p.y,C->p.z)).glFloat());      
  glVertex3f(C->p.x,C->p.y,C->p.z);
          
//  std::cout << Vector3(A->p.x,A->p.y,A->p.z);
//  printf("--> %f\n",A->p.x);

}

void Environ::makeEnvironGlList() {

  glNewList(ENVIRON_GLLIST_NUM,GL_COMPILE);

  GtsCartesianGrid g;
  GtsSurface * surface;
  gdouble iso = 0;
  GtsIsoCartesianFunc func = &(Environ::gtsFunction);
  
  g.nx = 70;
  g.ny = 50;
  g.nz = 70;
  
/*  g.nx = 7;
  g.ny = 5;
  g.nz = 7;*/


  /* interval is [-10:10][-10:10][-10:10] */
  g.x = -5.0; g.dx = 25./(gdouble) (g.nx - 1);
  g.y = -10.0; g.dy = 20./(gdouble) (g.ny - 1);
  g.z = 15.0; g.dz = 25./(gdouble) (g.nz - 1);

  surface = gts_surface_new (gts_surface_class (),
			     gts_face_class (),
			     gts_edge_class (),
			     gts_vertex_class ());
			     
  gts_isosurface_tetra (surface, g, func, NULL, iso);

  gts_surface_print_stats (surface, stdout);
//  gts_surface_write (surface, stdout);

//  GtsSurfaceTraverse* trav = gts_surface_traverse_new(surface,NULL);
//  GtsFace* face = gts_surface_traverse_next(trav,NULL);


  glBegin(GL_TRIANGLES);

  gts_surface_foreach_face(surface, (GtsFunc) print_vertex, this);

  glEnd();



/*  
  GSList *strip = gts_surface_strip(surface);
  while(strip) {
  
        glBegin(GL_TRIANGLE_STRIP);  
  
  	std::cout << "strip";  
	Environ* env = this;
  	GSList* face = (GSList*) strip->data;
	
	if (!face) continue;
	assert(face);

	GtsVertex *A,*B,*C;  	
	gts_triangle_vertices(&( ((GtsFace*)face->data)->triangle),&A,&B,&C);
	Vector3 last3 = Vector3(A->p.x,A->p.y,A->p.z);
	Vector3 last2 = Vector3(B->p.x,B->p.y,B->p.z);
	Vector3 last1 = Vector3(C->p.x,C->p.y,C->p.z);
	GtsTriangle* last = &((GtsFace*)face->data)->triangle;
	
	std::cout << Vector3(A->p.x,A->p.y,A->p.z);
	std::cout << Vector3(B->p.x,B->p.y,B->p.z);
	std::cout << Vector3(C->p.x,C->p.y,C->p.z) << std::endl;						
	std::cout.flush();
	
	glNormal3fv(env->gradient(last3).glFloat());    
	glVertex3f(A->p.x,A->p.y,A->p.z);
	glNormal3fv(env->gradient(last2).glFloat());    
	glVertex3f(B->p.x,B->p.y,B->p.z);
	glNormal3fv(env->gradient(last1).glFloat());    
	glVertex3f(C->p.x,C->p.y,C->p.z);
	
	face = face->next;
	
	while(face) {
//		print_vertex((GtsFace*)face->data, (void**)(&env) );

		GtsTriangle* current = &((GtsFace*)face->data)->triangle;
		
		GtsVertex *newP = gts_triangle_vertex_opposite(current,gts_triangles_common_edge(current,last));
		Vector3 newV = Vector3(newP->p.x,newP->p.y,newP->p.z);
		
		gts_triangle_vertices(&( ((GtsFace*)face->data)->triangle),&A,&B,&C);
		Vector3 new3 = Vector3(A->p.x,A->p.y,A->p.z);
		Vector3 new2 = Vector3(B->p.x,B->p.y,B->p.z);
		Vector3 new1 = Vector3(C->p.x,C->p.y,C->p.z);		
		
//		if ( !( newV == last1 || newV == last2 || newV == last3) ) {

		if ( (new1 == last1 && new2 == last2) || (new1 == last2 && new2 == last1) 
		   ||(new2 == last1 && new3 == last2) || (new2 == last2 && new3 == last1)
		   ||(new3 == last1 && new1 == last2) || (new3 == last2 && new1 == last1) ) {

			glNormal3fv(env->gradient(newV).glFloat());    			
			glVertex3fv(newV.glFloat());
			
			std::cout << "new: " << newV << std::endl;
		
			last3 = last2;
			last2 = last1;
			last1 = newV;	
			last = current;
		
		} else {
		
			glEnd();
			glBegin(GL_TRIANGLE_STRIP);

			gts_triangle_vertices(&( ((GtsFace*)face->data)->triangle),&A,&B,&C);
			last3 = Vector3(A->p.x,A->p.y,A->p.z);
			last2 = Vector3(B->p.x,B->p.y,B->p.z);
			last1 = Vector3(C->p.x,C->p.y,C->p.z);
			
			std::cout << last3;
			std::cout << last2;
			std::cout << last1 << std::endl;						
			
			
			last = &((GtsFace*)face->data)->triangle;
	
			glNormal3fv(env->gradient(last3).glFloat());    
			glVertex3f(A->p.x,A->p.y,A->p.z);
			glNormal3fv(env->gradient(last2).glFloat());    
			glVertex3f(B->p.x,B->p.y,B->p.z);
			glNormal3fv(env->gradient(last1).glFloat());    
			glVertex3f(C->p.x,C->p.y,C->p.z);



		}		
		face = face->next;
	}

	glEnd();
  	strip = strip->next;
  
  }  
*/

  glEndList();

  delete(surface);

}


void Environ::glVisualize() const {

 float diffuse[] = { 1.0, 1.0, 1.0,1.0};
 glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse);


  glCallList(ENVIRON_GLLIST_NUM);

}
		



