
#include "object.h"
#include "lists.h"
#include "environ.h"

//#define EULER
#define LEAPFROG
/*
void Object::setLists(PointList* p, ConnectivityList* c, SpringList *s) {

	delete points;
	delete conn;
	delete springs;
	points = p;
	conn = c;
	springs = s;

}
*/

void Object::initSimulation(FLT dt) {

#ifdef LEAPFROG
	PointList::updateAllVelocities(-dt);
#endif

}

void Object::simulate(FLT dt) {


#ifdef EULER

	// Euler:
	
	  // calculate F(t)
	points->setExternalForces();
	springs->generateSpringForces();

	  // x(t+dt) = x(t) + dt * v(t)
	points->updateAllPositions(dt);
	
	  // v(t+dt) = v(t) + dt * a(t)
	points->updateAllVelocities(dt);
#endif	

#ifdef LEAPFROG	
	// Leap Frog

	  // calculate F(t)
	PointList::setExternalForces();
	SpringList::generateSpringForces();

	  // v(t+dt) = v(t) + dt * a(t)
	PointList::updateAllVelocities(dt);

	  // x(t+dt) = x(t) + dt * v(t)
	PointList::updateAllPositions(dt);
#endif
	
	
//	points->debug();
	
	/* detect & handle collision with environement */
	
	for(unsigned int i=0; i<Nsurfacepoints; i++) {
	
		MassPoint& p = surfacepoints[i];
		
		FLT pen = env.inside(p);
		if (pen<-2*INTERSECTION_TIME_EPSILON) {
		
//			std::cout << "hit " << pen ;
		
			// inside object, project back onto surface			
			env.project(p);
			
			// damp velocity
			p.v.scale(0.3);
			if (p.v.length()<0.05) {
			
				p.v = Vector3(0);
				
			} else {
			
				// mirror velocity
				Vector3 n = env.gradient(p);
			
				FLT x = -2*Vector3::dotProd(n,p.v);
				Vector3 reflecteddirection = n.scaled(x);
				reflecteddirection.add(p.v);
				p.v = reflecteddirection;
				
			}
			
//			std::cout << "-->" << env.inside(p);
					
		}
		
	
	}

}

void Object::setV(const Vector3& v) {

	for(unsigned int i = 0; i< Nsurfacepoints; i++) {		
		surfacepoints[i].v.set(v);
	}
	for(unsigned int i = 0; i< Ninnerpoints; i++) {		
		innerpoints[i].v.set(v);
	}		

}


Object::~Object() {
}
