#include <stdlib.h>
#include <GL/glut.h>
#include <time.h>
#include <math.h>
#include <stdio.h>
#include <iostream>
#include <time.h>

#include<unistd.h>

#include "springbox.h"
#include "environ.h"
#include "sphere.h"

// global variables

MassPoint* surfacepoints;
MassPoint* innerpoints;

int Nsurfacepoints;
int Ninnerpoints;

Spring* springs;
int Nsprings;

int* connectivity;
int Nconnectivity;

//

//SpringBox* box;
Object* obj;
Environ* environement;

// 

float t_global = 0.0;
clock_t t_real_init = 0;
float t_last_update;
float t_deltat = 0.0001;
int t_framecount = 0;
float t_lastdisplay = -10;

const float FRAME_RATE_UPDATE_INTERVAL = 0.1;
const float DISPLAY_FRAMERATE = 25;

float Zoom=30;

int render_mode = 2;
int object_type = 1;
int environ_type = ENVIRON_XYZ;

extern int inelastic_springs;

static float front_shininess[] =   {60.0};
static float front_specular[]  =   { 0.7, 0.7, 0.7, 1.0 };
static float ambient[]         =   { 0.1, 0.1, 0.1, 1.0 };
static float diffuse[]         =   { 1.0, 1.0, 1.0, 1.0 };
static float position0[]       =   { 4.0, 0.0, 10.0, 0.0 };
static float position1[]       =   {-112.0,-1.0, 21.0, 0.0 };
static float lmodel_ambient[]  =   { 0.5, 0.5, 0.5, 1.0 };
static float lmodel_twoside[]  =   {GL_FALSE};       

static float fogcolor[4]={.65,.7,1.0,1.0};    



static const int TOGGLE_DRAW_SPRINGS = 1;
static const int TOGGLE_DRAW_FACES = 2;
static const int TOGGLE_INELASTIC_SPRINGS = 3;

static const int OBJECT_TYPE_CUBE      = 1;
static const int OBJECT_TYPE_LONGBOX   = 2;
static const int OBJECT_TYPE_RANDOMBOX = 3;
static const int OBJECT_TYPE_SPHERE    = 4;




/*  Initialize material property and light source.
 */
void myinit (void)
{
  glClearDepth(1.0);
  glClearColor( fogcolor[0]*.8,fogcolor[1]*.8,fogcolor[2],fogcolor[3] );
  glColor3f( 0, 1.0, .0 );

  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
  glFlush();
  glutSwapBuffers(); 
           	
  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, position0);
  glLightfv(GL_LIGHT1, GL_AMBIENT, ambient);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuse);
  glLightfv(GL_LIGHT1, GL_POSITION, position1);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside);
//  glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER,0);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
//  glEnable(GL_LIGHT1);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_NORMALIZE);     
	
  glMaterialfv(GL_FRONT, GL_SHININESS, front_shininess);
  glMaterialfv(GL_FRONT, GL_SPECULAR, front_specular);
  
  
	float ambientScene[] = {0.1,0.1,.1,1.0};
	glMaterialfv(GL_FRONT,GL_AMBIENT,ambientScene);


  glShadeModel( GL_SMOOTH ); 
//  glEnable(GL_NORMALIZE);
   
}


//////////////////////////////***** DISPLAY ****************//////////////////////////

void display ()
{

//	glPushMatrix();

//	std::cout << "X.";
//	std::cout.flush();

	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // noetig??
	
	t_lastdisplay  = (clock()-t_real_init)/1000000.0f;	

	obj->glVisualize(render_mode);
	environement->glVisualize();
 	  
	glutSwapBuffers();       


	
//	glPopMatrix();	
}

void myReshape(int w, int h)
{
   glViewport (0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
   gluPerspective(Zoom, (GLfloat) w/(GLfloat) h, 1, 50.0);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   gluLookAt(0,1.5,0,  1,1,3  ,0,1,0);

 //  gluLookAt (90.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);         
}

void keyboard (unsigned char key, int x, int y)
{

	if (key == 'q') exit(0);

}

Object* createObject() {

	switch(object_type) {
	
		case OBJECT_TYPE_SPHERE: 
			return new Sphere(*environement,Vector3(7,1,18),2,50,3000);
			break;
			
		case OBJECT_TYPE_CUBE: {
			int res[3] = {5,5,5};		
			return new SpringBox(1,*environement,Vector3(3,0,18),Vector3(6,3,21),res,50,2500);
			break;}
			
		case OBJECT_TYPE_LONGBOX: {
			int res[3] = {15,4,4};		
			return new SpringBox(0,*environement,Vector3(3,0,18),Vector3(10,1,19),res,50,6500);			
			break; }
			
		case OBJECT_TYPE_RANDOMBOX: {
			int res[3] = {(int)(drand48()*7)+1,(int)(drand48()*7)+1,(int)(drand48()*7)+1};
			return new SpringBox(0,*environement,Vector3(3,0,18),Vector3(3+res[0]*0.7,0+res[1]*0.7,18+res[2]*0.7),res,0.5*(res[0]+1)*(res[1]+1)*(res[2]+1),7000);	
			break; }
		
	
	}
	
	
	return 0;


}



void scene_init() {

	surfacepoints = 0;
	innerpoints = 0;
	springs = 0;
	connectivity = 0;

//	int res[3] = {5,5,5};
	environement = new Environ();
	obj = createObject();
	//new SpringBox(0,*environement,Vector3(3,0,18),Vector3(6,3,21),res,30,1000);
//	obj->setV(Vector3(1+drand48()*5,6,1+drand48()*5));
	obj->setV(Vector3(3.95467,6,3.00705));

	// for leap frog init:
	obj->initSimulation(0.02/100);

}

void reset_scene() {

//	int res[3] = {15,4,4};
//	int res[3] = {,1,1};
	delete(obj);
	obj = createObject();
	//new Sphere(*environement,Vector3(7,1,18),2,5,1000);
	//new SpringBox(0,*environement,Vector3(3,0,18),Vector3(10,1,19),res,50,5000);
	Vector3 bv = Vector3(1+drand48()*10,6,1+drand48()*5);
//	std::cout << "Box velocity = " << bv << std::endl;
	obj->setV(bv);

	t_lastdisplay = -10;
	
	// for leap frog init:
	obj->initSimulation(0.02/100);

}

void doSimulationStep() {
	
	if (t_global>10) {
		reset_scene();
		t_global = 0;
		t_real_init = clock();
		t_last_update = 0;
	}	
	
	t_framecount++;
	
	float realtime = (clock()-t_real_init)/1000000.0f;
	
	if (t_last_update + FRAME_RATE_UPDATE_INTERVAL < realtime) {
	
		// adapt timestep
		t_deltat = 2*FRAME_RATE_UPDATE_INTERVAL / t_framecount;
//		std::cout << "realtime = " << realtime << "  simulation time = "<<t_global;	
//		std::cout << "adapting: new dt = " << t_deltat << std::endl;
		t_global = realtime;		
		t_framecount = 0;	
		t_last_update = realtime;
	}
	
	if (t_global > realtime + 0.05) {
	
		do{
			realtime = (clock()-t_real_init)/1000000.0f;		
		
		} while (t_global > realtime +0.01);
	
	}

	if (t_deltat > 0.01) t_deltat = 0.01;
	
	t_global += t_deltat;

	int N = 5;
	for(int i=0; i<N; i++)	
		obj->simulate(t_deltat/N);	

}


static void idle_( void )
{
	doSimulationStep();
	float realtime = (clock()-t_real_init)/1000000.0f;
	if (t_lastdisplay + 1.0/DISPLAY_FRAMERATE < realtime) {
	
	  	glutPostRedisplay();
		
	}
	
}     

static void objecttype_chooser(int value) {

	object_type = value;

}

static void environtype_chooser(int value) {

	environ_type = value;
	Environ* newenv = new Environ();
	Environ* oldenv = environement;
	environement = newenv;
	delete(oldenv);
	reset_scene();

}


static void mainmenu(int value) {

	switch(value) {

		case TOGGLE_DRAW_SPRINGS:
			render_mode ^= 1;
			break;
			
		case TOGGLE_DRAW_FACES:
			render_mode ^= 2;
			break;
			
		case TOGGLE_INELASTIC_SPRINGS:
			if (inelastic_springs == 0) 
				inelastic_springs = 1;
			else
				inelastic_springs = 0;
			break;
					

	}

}
 
#define VIS
 
int main(int argc, char** argv)
{


	srand48((unsigned)time(NULL));


	printf("scene initialized.\n");
#ifdef VIS	
   	glutInit(&argc, argv);
	glutInitWindowSize (500, 500);
	glutInitWindowPosition(100,100);    
	glutInitDisplayMode( GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB ); 

	glutCreateWindow (argv[0]);
#endif	
	
#ifdef VIS
	myinit ();	
#endif	

	// object menu
	int objecttype_menu = glutCreateMenu(objecttype_chooser);
	char* omentry1 = "Sphere";
	glutAddMenuEntry(omentry1,OBJECT_TYPE_SPHERE);
	char* omentry2 = "Cube";
	glutAddMenuEntry(omentry2,OBJECT_TYPE_CUBE);
	char* omentry3 = "Stick";
	glutAddMenuEntry(omentry3,OBJECT_TYPE_LONGBOX);
	char* omentry4 = "Random Box";
	glutAddMenuEntry(omentry4,OBJECT_TYPE_RANDOMBOX);
	
	// environ menu
	int environtype_menu = glutCreateMenu(environtype_chooser);
	char* ementry1 = "xyz";
	glutAddMenuEntry(ementry1,ENVIRON_XYZ);
	char* ementry2 = "sinx";
	glutAddMenuEntry(ementry2,ENVIRON_SINX);
	char* ementry3 = "sinxsinz";
	glutAddMenuEntry(ementry3,ENVIRON_SINXSINZ);
	char* ementry4 = "sinradial";
	glutAddMenuEntry(ementry4,ENVIRON_SINXXZZ);
	
	
	// main menu
	int main_menu = glutCreateMenu(mainmenu);
	
	char* mentry1 = "toggle draw springs";
	glutAddMenuEntry(mentry1,TOGGLE_DRAW_SPRINGS);
	char* mentry2 = "toggle draw faces";
	glutAddMenuEntry(mentry2,TOGGLE_DRAW_FACES);
	char* mentry5 = "toggle inelasticity";
	glutAddMenuEntry(mentry5,TOGGLE_INELASTIC_SPRINGS);	
	
	char* mentry3 = "object type";
	glutAddSubMenu(mentry3,objecttype_menu);
	char* mentry4 = "env type";
	glutAddSubMenu(mentry4,environtype_menu);
		
	glutAttachMenu(GLUT_RIGHT_BUTTON);

	glutFullScreen();

	scene_init();
	
#ifndef VIS	
for(;;)	doSimulationStep();
#endif	
	t_real_init = clock();
	t_last_update = 0;

#ifdef VIS

	glutReshapeFunc (myReshape);
	glutDisplayFunc(display); 
	glutIdleFunc( idle_ );  
	glutKeyboardFunc(keyboard);
	glutMainLoop(); 
#endif	    

}
