#include "sphere.h"
#include "lists.h"
#include "masspoint.h"

#include <fstream>

#include <GL/glut.h>

extern MassPoint* surfacepoints;
extern MassPoint* innerpoints;

extern int Nsurfacepoints;
extern int Ninnerpoints;

extern Spring* springs;
extern int Nsprings;

extern int* connectivity;
extern int Nconnectivity;




void Sphere::makeSpring(int i1, int i2, FLT k, int& ci) {

	assert(i1>=0);
	assert(i2>=0);
	
	assert(ci<Nsprings);
	springs[ci] = Spring(i1,i2,k);
	ci++;

}

void Sphere::makeTriangle(int e1, int e2, int e3, int& fi, bool flip) {

	assert(e1>=0);
	assert(e2>=0);
	assert(e3>=0);
	assert(fi<3*Nconnectivity);

	if (flip == false) {	
		connectivity[3*fi]   = e1;
		connectivity[3*fi+1] = e2;
		connectivity[3*fi+2] = e3;
	} else {
		connectivity[3*fi+1]   = e1;
		connectivity[3*fi] = e2;
		connectivity[3*fi+2] = e3;	
	}
	
	fi++;

}

using namespace std;

Sphere::Sphere(const Environ& env, const Vector3& midpoint, FLT radius, FLT totalmass, FLT springk):
  Object(env) {
  
  	FLT m_i = totalmass / 84;
	
	// calculate array sizes
	
	delete(surfacepoints);	
	Nsurfacepoints = 84;
	surfacepoints = new MassPoint[Nsurfacepoints];
	
	delete(innerpoints);
	Ninnerpoints = 0;
	innerpoints = 0;
	
	delete(springs);
	Nsprings =  480;
	springs = new Spring[Nsprings];
	
	delete(connectivity);
	Nconnectivity = 80;
	connectivity = new int[3*Nconnectivity];

	int ci=0,fi=0,pi=0;
	
	ifstream fin("sphere.vertexes");
	while(!fin.eof()) {
	
		char tmp;
	
		FLT px, py ,pz;
		fin >> px;
		fin >> py;
		fin >> pz;
		fin >> tmp;
		
		std::cout << pi << Vector3(px,py,pz) << std::endl;
		
		Vector3 p(px,py,pz);
		p.scale(radius);
		p.add(midpoint);
	
		surfacepoints[pi++] = MassPoint(p,m_i);
	
	}	
	fin.close();
	
{	ifstream fin("sphere.springs");
	while(!fin.eof()) {
	
		int i1,i2,tmp;
		char k;
		fin >> i1;
		fin >> k;
		fin >> i2;
		fin >> k;
		fin >> tmp;
		fin >> k;
		
		std::cout << ci << "/" << i1 << "," << i2 << std::endl;	
		makeSpring(i1,i2,springk,ci);
	
	}	
	fin.close();
}	

{	ifstream fin("sphere.faces");
	while(!fin.eof()) {
	
		int i1,i2,i3,tmp;
		char k;
		fin >> i1;
		fin >> k;
		fin >> i2;
		fin >> k;
		fin >> i3;
		fin >> k;		
		fin >> tmp;
		fin >> k;
		
		std::cout << fi << "/" << i1 << "," << i2 <<"," << i3 << std::endl;	
		makeTriangle(i1,i2,i3,fi,false);
	
	}	
	fin.close();
}	

	
/*		
		
		
	int i = 0;
	int ci = 0;
	int fi = 0;

	for(int x=0; x<=r[0]; x++) {
		
		for(int y=0; y<=r[1]; y++) {
		
			for(int z=0; z<=r[2]; z++) {
			
				MassPoint p( e1[0]+(e2[0]-e1[0])/r[0]*x,
				           e1[1]+(e2[1]-e1[1])/r[1]*y, 
				           e1[2]+(e2[2]-e1[2])/r[2]*z, m_i);
					   
				surfacepoints[i] = p;

				if (x != 0) {	// edges springs in x-direction
					makeSpring(i,i-(r[1]+1)*(r[2]+1),springk,ci);
				}
				if (y != 0) {   // edges springs in y-direction
					makeSpring(i,i-(r[2]+1),springk,ci);
				}
				if (z != 0) {   // edges springs in z-direction
					makeSpring(i,i-1,springk,ci);
				}
				if (type&1) {
					// cross springs
					if ( (x != 0) && (y != 0) && (z != 0) ) {					
						makeSpring(i,i-(r[1]+1)*(r[2]+1)-(r[2]+1)-1,springk,ci);				
					}
					if ( (x != 0) && (y != r[1]) && (z != 0) ) {
						makeSpring(i,i-(r[1]+1)*(r[2]+1)+(r[2]+1)-1,springk,ci);
					}
					if ( (x != 0) && (y != r[1]) && (z != r[2]) ) {
						makeSpring(i,i-(r[1]+1)*(r[2]+1)+(r[2]+1)+1,springk,ci);
					}
					if ( (x != 0) && (y != 0) && (z != r[2]) ) {
						makeSpring(i,i-(r[1]+1)*(r[2]+1)-(r[2]+1)+1,springk,ci);
					}
				}
				// side diagonals
				if (x != 0 && y != r[1]) {
					makeSpring(i,i-(r[1]+1)*(r[2]+1)+(r[2]+1),springk,ci);
				}
				if (x != 0 && y != 0) {
					makeSpring(i,i-(r[1]+1)*(r[2]+1)-(r[2]+1),springk,ci);								
				}
				if (x != 0 && z != r[2]) {
					makeSpring(i,i-(r[1]+1)*(r[2]+1)+1,springk,ci);
				}
				if (x != 0 && z != 0) {
					makeSpring(i,i-(r[1]+1)*(r[2]+1)-1,springk,ci);		
				}
				if (y != 0 && z != 0) {
					makeSpring(i,i-(r[2]+1)-1,springk,ci);												
				}
				if (y != 0 && z != r[2]) {
					makeSpring(i,i-(r[2]+1)+1,springk,ci);												
				}
				
				// faces
				if ( (z == 0 || z == r[2]) && y != r[1] && x !=0) {				
					makeTriangle(i, i-(r[1]+1)*(r[2]+1), i-(r[1]+1)*(r[2]+1)+(r[2]+1),fi, z == r[2]);
				}
				if ( (z == 0 || z == r[2]) && y != 0 && x !=0) {				
					makeTriangle(i, i-(r[1]+1)*(r[2]+1), i-(r[2]+1),fi, z == 0);
				}
				if ( (y == 0 || y == r[1]) && x !=0 && z != r[2] ) {
					makeTriangle(i, i-(r[1]+1)*(r[2]+1), i-(r[1]+1)*(r[2]+1)+1,fi, y == 0);
				}
				if ( (y == 0 || y == r[1]) && x !=0 && z != 0 ) {
					makeTriangle(i, i-(r[1]+1)*(r[2]+1), i-1,fi,y == r[1]);	
				}
				if ( (x == 0 || x == r[0]) && y != 0 && z != r[2] ) {
					makeTriangle(i, i-(r[2]+1), i-(r[2]+1)+1,fi, x == r[0]);
				}
				if ( (x == 0 || x == r[0]) && y != 0 && z != 0 ) {
					makeTriangle(i, i-(r[2]+1), i-1,fi, x == 0);
				}

				
				
				i++;
			
			}
		
		}
	
	
	}*/

	assert(ci == Nsprings);	
	assert(fi == Nconnectivity);
	assert(pi == Nsurfacepoints);
	
}


void Sphere::glVisualize(int mode) {

/*
    if (mode&1) {

	glBegin(GL_LINES);

	
	for(int i=0; i<Nsprings; i++) {

		glVertex3fv(springs[i].p1->glFloat());
		glVertex3fv(springs[i].p2->glFloat());
	
	}
	
	glEnd();	

    }
    
    if (mode&2) {


	glBegin(GL_TRIANGLES);
	
	for(int i=0; i<Nconnectivity; i++) {
	
		Vector3* e1 = &surfacepoints[connectivity[3*i]];
		Vector3* e2 = &surfacepoints[connectivity[3*i+1]];
		Vector3* e3 = &surfacepoints[connectivity[3*i+2]];		
		
		Vector3 s1 = Vector3::sub(*e1,*e2);
		Vector3 s2 = Vector3::sub(*e1,*e3);
		Vector3 n = Vector3::crossProd(s1,s2);
		n.normalize();
		glNormal3fv( n.glFloat() );
		
		glVertex3fv( e1->glFloat() );
		glVertex3fv( e2->glFloat() );
		glVertex3fv( e3->glFloat() );

	
	}

	glEnd();


    }
    */
    
    if (mode&1) {

    	float diffuse[] = { .0, 0, 0, 1.0};
    	glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse);

	glBegin(GL_LINES);

	for(int i=0; i<Nsprings; i++) {
	
		FLT force = fabs( 1 - springs[i].li / springs[i].lc );
		float ambient[] = {5*force,40*force,1, 1.0-3*force};
		glMaterialfv(GL_FRONT,GL_AMBIENT,ambient);	

		glVertex3fv(springs[i].p1->glFloat());
		glVertex3fv(springs[i].p2->glFloat());
	
	}
	
	glEnd();	
	
	float ambientScene[] = {0.1,0.1,.1,1.0};
	glMaterialfv(GL_FRONT,GL_AMBIENT,ambientScene);


    }
    
    if (mode&2) {
    
    
    	float diffuse[] = { 1.0, 0.3, 0.1,1.0};
    	glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse);


	glBegin(GL_TRIANGLES);
	
	for(int i=0; i<Nconnectivity; i++) {
	
		Vector3* e1 = &surfacepoints[connectivity[3*i]];
		Vector3* e2 = &surfacepoints[connectivity[3*i+1]];
		Vector3* e3 = &surfacepoints[connectivity[3*i+2]];		
		
		Vector3 s1 = Vector3::sub(*e1,*e2);
		Vector3 s2 = Vector3::sub(*e1,*e3);
		Vector3 n = Vector3::crossProd(s1,s2);
		n.normalize();
		glNormal3fv( n.glFloat() );
		
/*		glArrayElement(connectivity[3*i]);
		glArrayElement(connectivity[3*i+1]);
		glArrayElement(connectivity[3*i+2]);*/
		
		glVertex3fv( e1->glFloat() );
		glVertex3fv( e2->glFloat() );
		glVertex3fv( e3->glFloat() );

	
	}

	glEnd();


    }
    

}

