#include "springbox.h"
#include "lists.h"
#include "masspoint.h"

#include <GL/glut.h>

extern MassPoint* surfacepoints;
extern MassPoint* innerpoints;

extern int Nsurfacepoints;
extern int Ninnerpoints;

extern Spring* springs;
extern int Nsprings;

extern int* connectivity;
extern int Nconnectivity;


void SpringBox::makeSpring(int i1, int i2, FLT k, int& ci) {

	assert(i1>=0);
	assert(i2>=0);

//	MassPoint* p1 = &(surfacepoints[i1]);
//	MassPoint* p2 = &(surfacepoints[i2]);
	
	assert(ci<Nsprings);
	springs[ci] = Spring(i1,i2,k);
	ci++;

}

void SpringBox::makeTriangle(int e1, int e2, int e3, int& fi, bool flip) {

	assert(e1>=0);
	assert(e2>=0);
	assert(e3>=0);
	assert(fi<3*Nconnectivity);

	if (flip == false) {	
		connectivity[3*fi]   = e1;
		connectivity[3*fi+1] = e2;
		connectivity[3*fi+2] = e3;
	} else {
		connectivity[3*fi+1]   = e1;
		connectivity[3*fi] = e2;
		connectivity[3*fi+2] = e3;	
	}
	
	fi++;

}

SpringBox::SpringBox(int type, const Environ& env, const Vector3& e1, const Vector3& e2, int r[3], FLT totalmass, FLT springk):
  Object(env) {
  
  	FLT m_i = totalmass / (r[0]*r[1]*r[2]);
	
	// calculate array sizes
	
	delete(surfacepoints);	
	Nsurfacepoints = (r[0]+1)*(r[1]+1)*(r[2]+1);
	surfacepoints = new MassPoint[Nsurfacepoints];
	
	delete(innerpoints);
	Ninnerpoints = 0;
	innerpoints = 0;
	
	delete(springs);
	Nsprings =   r[0]*(r[1]+1)*(r[2]+1)   // edges of the cube (x)
	           + (r[0]+1)*r[1]*(r[2]+1)   // edges of the cube (y)
		   + (r[0]+1)*(r[1]+1)*r[2]   // edges of the cube (z)
		   + 2*r[0]*r[1]*(r[2]+1)     // side springs
		   + 2*r[0]*(r[1]+1)*r[2]     // side springs
		   + 2*(r[0]+1)*r[1]*r[2]     // side springs
		   ;
		   
	if (type&1) 
		Nsprings += 4*r[0]*r[1]*r[2]; // diagonal springs
		   
	springs = new Spring[Nsprings];
	
	delete(connectivity);
	Nconnectivity = 2*( 2*r[0]*r[1]+2*r[0]*r[2]+2*r[1]*r[2] );
	connectivity = new int[3*Nconnectivity];
		
	int i = 0;
	int ci = 0;
	int fi = 0;

	for(int x=0; x<=r[0]; x++) {
		
		for(int y=0; y<=r[1]; y++) {
		
			for(int z=0; z<=r[2]; z++) {
			
				MassPoint p( e1[0]+(e2[0]-e1[0])/r[0]*x,
				           e1[1]+(e2[1]-e1[1])/r[1]*y, 
				           e1[2]+(e2[2]-e1[2])/r[2]*z, m_i);
					   
				surfacepoints[i] = p;

				if (x != 0) {	// edges springs in x-direction
					makeSpring(i,i-(r[1]+1)*(r[2]+1),springk,ci);
				}
				if (y != 0) {   // edges springs in y-direction
					makeSpring(i,i-(r[2]+1),springk,ci);
				}
				if (z != 0) {   // edges springs in z-direction
					makeSpring(i,i-1,springk,ci);
				}
				if (type&1) {
					// cross springs
					if ( (x != 0) && (y != 0) && (z != 0) ) {					
						makeSpring(i,i-(r[1]+1)*(r[2]+1)-(r[2]+1)-1,springk,ci);				
					}
					if ( (x != 0) && (y != r[1]) && (z != 0) ) {
						makeSpring(i,i-(r[1]+1)*(r[2]+1)+(r[2]+1)-1,springk,ci);
					}
					if ( (x != 0) && (y != r[1]) && (z != r[2]) ) {
						makeSpring(i,i-(r[1]+1)*(r[2]+1)+(r[2]+1)+1,springk,ci);
					}
					if ( (x != 0) && (y != 0) && (z != r[2]) ) {
						makeSpring(i,i-(r[1]+1)*(r[2]+1)-(r[2]+1)+1,springk,ci);
					}
				}
				// side diagonals
				if (x != 0 && y != r[1]) {
					makeSpring(i,i-(r[1]+1)*(r[2]+1)+(r[2]+1),springk,ci);
				}
				if (x != 0 && y != 0) {
					makeSpring(i,i-(r[1]+1)*(r[2]+1)-(r[2]+1),springk,ci);								
				}
				if (x != 0 && z != r[2]) {
					makeSpring(i,i-(r[1]+1)*(r[2]+1)+1,springk,ci);
				}
				if (x != 0 && z != 0) {
					makeSpring(i,i-(r[1]+1)*(r[2]+1)-1,springk,ci);		
				}
				if (y != 0 && z != 0) {
					makeSpring(i,i-(r[2]+1)-1,springk,ci);												
				}
				if (y != 0 && z != r[2]) {
					makeSpring(i,i-(r[2]+1)+1,springk,ci);												
				}
				
				// faces
				if ( (z == 0 || z == r[2]) && y != r[1] && x !=0) {				
					makeTriangle(i, i-(r[1]+1)*(r[2]+1), i-(r[1]+1)*(r[2]+1)+(r[2]+1),fi, z == r[2]);
				}
				if ( (z == 0 || z == r[2]) && y != 0 && x !=0) {				
					makeTriangle(i, i-(r[1]+1)*(r[2]+1), i-(r[2]+1),fi, z == 0);
				}
				if ( (y == 0 || y == r[1]) && x !=0 && z != r[2] ) {
					makeTriangle(i, i-(r[1]+1)*(r[2]+1), i-(r[1]+1)*(r[2]+1)+1,fi, y == 0);
				}
				if ( (y == 0 || y == r[1]) && x !=0 && z != 0 ) {
					makeTriangle(i, i-(r[1]+1)*(r[2]+1), i-1,fi,y == r[1]);	
				}
				if ( (x == 0 || x == r[0]) && y != 0 && z != r[2] ) {
					makeTriangle(i, i-(r[2]+1), i-(r[2]+1)+1,fi, x == r[0]);
				}
				if ( (x == 0 || x == r[0]) && y != 0 && z != 0 ) {
					makeTriangle(i, i-(r[2]+1), i-1,fi, x == 0);
				}

				
				
				i++;
			
			}
		
		}
	
	
	}

	assert(ci == Nsprings);	
	assert(fi == Nconnectivity);
	
	glEnable(GL_VERTEX_ARRAY);
	glVertexPointer(3,GL_FLOAT,40,surfacepoints);

}


void SpringBox::glVisualize(int mode) {


    if (mode&1) {

    	float diffuse[] = { .0, 0, 0, 1.0};
    	glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse);

	glBegin(GL_LINES);

	for(int i=0; i<Nsprings; i++) {
	
		FLT force = fabs( 1 - springs[i].li / springs[i].lc );
		float ambient[] = {5*force,40*force,1, 1.0-3*force};
		glMaterialfv(GL_FRONT,GL_AMBIENT,ambient);	

		glVertex3fv(springs[i].p1->glFloat());
		glVertex3fv(springs[i].p2->glFloat());
	
	}
	
	glEnd();	
	
	float ambientScene[] = {0.1,0.1,.1,1.0};
	glMaterialfv(GL_FRONT,GL_AMBIENT,ambientScene);


    }
    
    if (mode&2) {
    
    
    	float diffuse[] = { 1.0, 0.3, 0.1,1.0};
    	glMaterialfv(GL_FRONT,GL_DIFFUSE,diffuse);


	glBegin(GL_TRIANGLES);
	
	for(int i=0; i<Nconnectivity; i++) {
	
		Vector3* e1 = &surfacepoints[connectivity[3*i]];
		Vector3* e2 = &surfacepoints[connectivity[3*i+1]];
		Vector3* e3 = &surfacepoints[connectivity[3*i+2]];		
		
		Vector3 s1 = Vector3::sub(*e1,*e2);
		Vector3 s2 = Vector3::sub(*e1,*e3);
		Vector3 n = Vector3::crossProd(s1,s2);
		n.normalize();
		glNormal3fv( n.glFloat() );
		
/*		glArrayElement(connectivity[3*i]);
		glArrayElement(connectivity[3*i+1]);
		glArrayElement(connectivity[3*i+2]);*/
		
		glVertex3fv( e1->glFloat() );
		glVertex3fv( e2->glFloat() );
		glVertex3fv( e3->glFloat() );

	
	}

	glEnd();


    }

}

